#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2007 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
LDAP authentication plugin.
"""
from ldappas.interfaces import ILDAPAuthentication

from zope.app import zapi
from zope.app.security.interfaces import ILoginPassword
from zope.app.security.interfaces import IAuthenticatedGroup, IEveryoneGroup
from zope.component import getUtility
from zope.session.interfaces import ISession
from zope.security.checker import ProxyFactory

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.app.security import Principal
from schooltool.app.security import PersonContainerAuthenticationPlugin
from schooltool.person.interfaces import IPersonFactory


class LDAPAuthenticationPlugin(PersonContainerAuthenticationPlugin):

    @property
    def auth(self):
        return getUtility(ILDAPAuthentication)

    def _checkPlainTextPassword(self, username, password):
        return self.auth.authenticateCredentials({'login': username or '',
                                                  'password': password or ''})
    def authenticate(self, request):
        """Identify a principal for request.

        Retrieves the username and password from the session.
        """
        session = ISession(request)[self.session_name]
        if 'username' in session:
            self.restorePOSTData(request)
            return self.getPrincipal('sb.person.' + session['username'])

        # Try HTTP basic too
        creds = ILoginPassword(request, None)
        if creds:
            login = creds.getLogin()
            if self._checkPlainTextPassword(login, creds.getPassword()):
                return self.getPrincipal('sb.person.' + login)

    def setCredentials(self, request, username, password):
        if not self._checkPlainTextPassword(username, password):
            raise ValueError('bad credentials')
        session = ISession(request)[self.session_name]
        session['username'] = username

    def clearCredentials(self, request):
        session = ISession(request)[self.session_name]
        try:
            del session['username']
        except KeyError:
            pass

    def getPrincipal(self, id):
        """Get principal meta-data.

        Returns principals for groups and persons.
        """
        app = ISchoolToolApplication(None)
        if id.startswith(self.person_prefix):
            username = id[len(self.person_prefix):]
            p_info = self.auth.principalInfo(username)

            if p_info:
                if username in app['persons']:
                    person = app['persons'][username]
                else:
                    factory = getUtility(IPersonFactory)
                    person = factory(p_info.id, p_info.id)
                    app['persons'][username] = person
                principal = Principal(id, person.title,
                                      person=ProxyFactory(person))
                for group in person.groups:
                    group_principal_id = self.group_prefix + group.__name__
                    principal.groups.append(group_principal_id)
                authenticated = zapi.queryUtility(IAuthenticatedGroup)
                if authenticated:
                    principal.groups.append(authenticated.id)
                everyone = zapi.queryUtility(IEveryoneGroup)
                if everyone:
                    principal.groups.append(everyone.id)
                return principal

        if id.startswith(self.group_prefix):
            group_name = id[len(self.group_prefix):]
            if group_name in app['groups']:
                group = app['groups'][group_name]
                # Group membership is not supported in SB, so we don't bother
                # filling in principal.groups.
                return Principal(id, group.title)
        return None
